@@ -1,10 +1,12 @@
 # -*- coding: utf-8 -*-
 from time      import time
 from threading import Lock as ThreadLock
+from tori.graph import DependencyNode as BaseDependencyNode, DependencyManager
 from tori.db.common    import Serializer, PseudoObjectId, ProxyObject
 from tori.db.entity    import BasicAssociation
 from tori.db.exception import UOWRepeatedRegistrationError, UOWUpdateError, UOWUnknownRecordError, IntegrityConstraintError
 from tori.db.mapper    import CascadingType
+from tori.db.metadata.helper import EntityMetadataHelper
 
 class Record(object):
     serializer = Serializer(0)
@@ -41,22 +43,16 @@ class Record(object):
 
         self.mark_as(Record.STATUS_CLEAN)
 
-class DependencyNode(object):
+class DependencyNode(BaseDependencyNode):
     """ Dependency Node
 
         This is designed to be bi-directional to maximize flexibility on
         traversing the graph.
     """
     def __init__(self, record):
-        self.created_at     = int(time() * 1000000)
-        self.record         = record
-        self.adjacent_nodes = set()
-        self.reverse_edges  = set()
-        self.walked         = False
+        super(DependencyNode, self).__init__()
 
-    def connect(self, other):
-        self.adjacent_nodes.add(other)
-        other.reverse_edges.add(self)
+        self.record = record
 
     @property
     def object_id(self):
@@ -66,17 +62,8 @@ class DependencyNode(object):
     def status(self):
         return self.record.status
 
-    @property
-    def score(self):
-        score = 0
-
-        for node in self.reverse_edges:
-            if node.status == Record.STATUS_DELETED:
-                continue
-
-            score += 1
-
-        return score
+    def _disavow_connection(self, node):
+        return node.status == Record.STATUS_DELETED
 
     def __eq__(self, other):
         return self.record.entity.__class__ == other.record.entity.__class__ and self.object_id == other.object_id
@@ -84,24 +71,9 @@ class DependencyNode(object):
     def __ne__(self, other):
         return self.record.entity.__class__ != other.record.entity.__class__ or self.object_id != other.object_id
 
-    def __lt__(self, other):
-        return self.score < other.score
-
-    def __le__(self, other):
-        return self.score <= other.score
-
-    def __gt__(self, other):
-        return self.score > other.score
-
-    def __ge__(self, other):
-        return self.score >= other.score
-
     def __hash__(self):
         return self.created_at
 
-    def __repr__(self):
-        return '<DependencyNode for {}, {}>'.format(self.object_id, self.score)
-
 class UnitOfWork(object):
     """ Unit of Work
 
@@ -160,7 +132,7 @@ class UnitOfWork(object):
             raise NonRefreshableEntity('The current record is not refreshable.')
 
         collection       = self._em.collection(entity.__class__)
-        updated_data_set = collection._api.find_one({'_id': entity.id})
+        updated_data_set = collection.driver.find_one(collection.name, {'_id': entity.id})
 
         # Reset the attributes.
         for attribute_name in updated_data_set:
@@ -301,11 +273,14 @@ class UnitOfWork(object):
         if isinstance(reference, ProxyObject):
             entity = reference._actual
 
-        if '__relational_map__' not in dir(entity):
+        if not EntityMetadataHelper.hasMetadata(entity):
             return
 
-        for property_name in entity.__relational_map__:
-            guide = entity.__relational_map__[property_name]
+        entity_meta = EntityMetadataHelper.extract(entity)
+        relational_map = entity_meta.relational_map
+
+        for property_name in relational_map:
+            guide = relational_map[property_name]
 
             if guide.inverted_by:
                 continue
@@ -391,7 +366,7 @@ class UnitOfWork(object):
 
     def _commit_changes(self, expected_class=None):
         # Load the sub graph of supervised collections.
-        for c in self._em.collections:
+        for c in self._em.repositories():
             if not c.has_cascading():
                 continue
 
@@ -430,31 +405,42 @@ class UnitOfWork(object):
             elif record.status == Record.STATUS_DELETED and commit_node.score > 0:
                 record.mark_as(Record.STATUS_CLEAN)
 
-    def _synchronize_new(self, collection, entity, change_set):
+    def _synchronize_new(self, repository, entity, change_set):
+        """ Synchronize the new / unsupervised data
+
+            :param repository: the target repository
+            :param entity: the entity
+            :param change_set: the change_set representing the entity
+        """
         pseudo_key = self._convert_object_id_to_str(entity.id, entity)
-        object_id  = collection._api.insert(change_set)
+        object_id  = repository.driver.insert(repository.name, change_set)
         entity.id  = object_id # update the entity ID
         actual_key = self._convert_object_id_to_str(object_id, entity)
 
         self._object_id_map[actual_key] = self._object_id_map[pseudo_key]
 
-    def _synchronize_update(self, collection, object_id, old_data_set, new_data_set):
-        """Synchronize the updated data
+    def _synchronize_update(self, repository, object_id, old_data_set, new_data_set):
+        """ Synchronize the updated data
 
-        :param collection: the target collection
-        :param object_id: the object ID
-        :param old_data_set: the original data (for event interception)
-        :param new_data_set: the updated data
+            :param repository: the target repository
+            :param object_id: the object ID
+            :param old_data_set: the original data (for event interception)
+            :param new_data_set: the updated data
         """
 
-        collection._api.update(
+        repository.driver.update(
+            repository.name,
             {'_id': object_id},
-            new_data_set,
-            upsert=False
+            new_data_set
         )
 
-    def _synchronize_delete(self, collection, object_id):
-        collection._api.remove({'_id': object_id})
+    def _synchronize_delete(self, repository, object_id):
+        """ Synchronize the deleted data
+
+            :param repository: the target repository
+            :param object_id: the object ID
+        """
+        repository.driver.remove(repository.name, {'_id': object_id})
 
     def _synchronize_records(self):
         writing_statuses = [Record.STATUS_NEW, Record.STATUS_DIRTY]
@@ -505,14 +491,7 @@ class UnitOfWork(object):
 
         # After constructing the dependency graph (as a supposedly directed acyclic
         # graph), do the topological sorting from the dependency graph.
-        final_order = []
-
-        for id in self._dependency_map:
-            node = self._dependency_map[id]
-
-            self._retrieve_dependency_order(node, final_order)
-
-        return final_order
+        return DependencyManager.get_order(self._dependency_map)
 
     def _compute_change_set(self, record):
         current_set = Record.serializer.encode(record.entity)
@@ -611,7 +590,7 @@ class UnitOfWork(object):
 
     def _load_extra_associations(self, record, change_set):
         origin_id      = record.entity.id
-        relational_map = record.entity.__relational_map__
+        relational_map = EntityMetadataHelper.extract(record.entity).relational_map
 
         for property_name in relational_map:
             if property_name not in change_set:
@@ -679,7 +658,8 @@ class UnitOfWork(object):
             cls = entity.__class__
 
         if cls:
-            class_hash = cls.__collection_name__
+            metadata   = EntityMetadataHelper.extract(cls)
+            class_hash = metadata.collection_name
 
         object_key = '{}/{}'.format(class_hash, str(object_id))
 
@@ -701,15 +681,14 @@ class UnitOfWork(object):
             if object_id not in self._dependency_map:
                 self._dependency_map[object_id] = DependencyNode(record)
 
+            relational_map = EntityMetadataHelper.extract(record.entity).relational_map
 
-
-            if not record.entity.__relational_map__:
-
+            if not relational_map:
                 continue
 
             # Go through the relational map to establish relationship between dependency nodes.
-            for property_name in record.entity.__relational_map__:
-                guide = record.entity.__relational_map__[property_name]
+            for property_name in relational_map:
+                guide = relational_map[property_name]
 
                 # Ignore a property from reverse mapping.
                 if guide.inverted_by:
